# -*- coding: utf-8 -*-
import numpy as np
from keras.models import model_from_json
from acumos.session import AcumosSession
from acumos.modeling import Model, List, create_namedtuple
from keras.preprocessing.image import load_img, img_to_array
from keras.applications.resnet50 import preprocess_input


# load keras model
print('Load the Keras model')
keras_model_name = "resnet50_notop"
json_file = open(keras_model_name+'.json', 'r')
keras_model_json = json_file.read()
json_file.close()
keras_model = model_from_json(keras_model_json)
keras_model.load_weights(keras_model_name+'_weights.h5')


print('Create Named Tuple')
# represents a flattened image
FlatImage = create_namedtuple('FlatImage', [('image', List[float]), ('shape', List[int])])
# represents a list of flattened images
FlatImages = create_namedtuple('FlatImages', [('images', List[FlatImage])])


def classify_doc(imgs: FlatImages) -> List[int]:

    model_input_ch = 3 # this is a constant for the model
    num_images = len(imgs.images)
    print('>Number of images:', num_images)

    h = int(imgs.images[0].shape[0])  # height
    w = int(imgs.images[0].shape[1])  # width
    print('>Images height and width: ' + str(h) + 'x' + str(w))

    imgs_batch = np.ndarray((num_images, h, w, model_input_ch))

    for k in range(num_images):
        img_l = list(imgs.images[k].image)
        ch = int(imgs.images[k].shape[2])  # channels
        print('>Image #'+str(k+1)+' -> number of channels: '+str(ch)+' - linearized length: '+str(len(img_l)))

        img_post = np.ndarray((h, w, model_input_ch))
        if ch == 1:
            print('>Image #'+str(k+1)+' -> replicating channel')
            for j in range(model_input_ch):
                img_post[:, :, j] = np.array(img_l, dtype=np.float32).reshape((h,w))
        else:
            print('>Image #'+str(k+1)+' -> reconstructing channels')
            for j in range(model_input_ch):
                img_post[:, :, j] = np.array(img_l[(h*w)*j:((h*w)*(j+1))], dtype=np.float32).reshape((h,w))

        print('>Image #'+ str(k+1) +' -> preprocessing for the model')
        imgs_batch[k,:,:,:] = preprocess_input(img_post)

    print('>Predicting the classes probabilities')
    predictions = keras_model.predict(imgs_batch)
    predicted_classes = []
    for k in range(num_images):
        predicted_classes.append(int(np.argmax(predictions[k,:])))

    return predicted_classes

# save the acumos model for onboarding
print('Saving Acumos Model')
model = Model(classify=classify_doc)
session = AcumosSession()
session.dump(model, 'VisualDocClassification', '.')  # creates ./model
print('Acumos Model Saved')
