# -*- coding: utf-8 -*-
import numpy as np
from keras.models import model_from_json
from acumos.session import AcumosSession
from acumos.modeling import Model, List, create_namedtuple



# load keras model
print('Load the Keras model')
keras_model_name = "resnet50_notop"
json_file = open(keras_model_name+'.json', 'r')
keras_model_json = json_file.read()
json_file.close()
keras_model = model_from_json(keras_model_json)
keras_model.load_weights(keras_model_name+'_weights.h5')


print('Create Named Tuple')

# represents a single "flattened" [1 x n] image channel
FlatChannel = create_namedtuple('FlatChannel', [('channel', List[float]), ('shape', List[int])])

# represents a collection of flattened image channels
FlatImage = create_namedtuple('FlatImage', [('image', List[FlatChannel])])


def classify_doc(img: FlatImage) -> List[float]:

    # recover the channels from the image
    print('>Extract each channel')
    img_r = list(img.image[0].channel)
    img_g = list(img.image[1].channel)
    img_b = list(img.image[2].channel)
    print('>Channel lenght:', len(img_r))

    # compute the width and height values
    print('>Dimensions: '+str(img.image[0].shape[0])+'x'+str(img.image[0].shape[1]))

    # reshape each channel (list) to a matrix (image) of rows/height (shape[0]) and cols/width (shape[1]) and cast to numpy type
    print('>Reshape each channel')
    img_r_rec = np.array(img_r, dtype=np.float32).squeeze().reshape((img.image[0].shape[0], img.image[0].shape[1]))
    img_g_rec = np.array(img_g, dtype=np.float32).squeeze().reshape((img.image[1].shape[0], img.image[1].shape[1]))
    img_b_rec = np.array(img_b, dtype=np.float32).squeeze().reshape((img.image[2].shape[0], img.image[2].shape[1]))
    print('>Channel dimension after reshape:', img_r_rec.shape)

    # reconstruct the original image [1 x H x W x C]
    print('>Reconstruct the three channels image')
    img_post = np.ndarray((1, img.image[0].shape[0], img.image[0].shape[1], 3))
    img_post[:, :, :, 0] = img_r_rec
    img_post[:, :, :, 1] = img_g_rec
    img_post[:, :, :, 2] = img_b_rec
    print('>Reconstructed image shape:',img_post.shape)

    # predict the classes probabilities
    print('>Classify the image content')
    predictions = keras_model.predict(img_post)
    print('>Predictions:', predictions.squeeze())

    return predictions

# save the acumos model for onboarding
print('Saving Acumos Model')
model = Model(classify=classify_doc)
session = AcumosSession()
session.dump(model, 'VisualDocClassification', '.')  # creates ./model
print('Acumos Model Saved')
