import requests
import numpy as np


classify_url = "http://127.0.0.1:3330/classify"


def classify_doc(img: np.ndarray):
    from visualdocclassification_6_pb2 import FlatChannel, FlatImage

    img_r = img[:, :, 0]
    img_g = img[:, :, 1]
    img_b = img[:, :, 2]

    # trasform each channel from a matrix to a list of float
    img_r_l = [float(i) for i in img_r.reshape(int(img_r.shape[0]) * int(img_r.shape[1]))]
    img_g_l = [float(i) for i in img_g.reshape(int(img_g.shape[0]) * int(img_g.shape[1]))]
    img_b_l = [float(i) for i in img_b.reshape(int(img_b.shape[0]) * int(img_b.shape[1]))]

    # create a FlatChannel for each channel
    ch_r = FlatChannel(channel=img_r_l, shape=[int(img_r.shape[0]), int(img_r.shape[1])])
    ch_g = FlatChannel(channel=img_g_l, shape=[int(img_g.shape[0]), int(img_g.shape[1])])
    ch_b = FlatChannel(channel=img_b_l, shape=[int(img_b.shape[0]), int(img_b.shape[1])])

    # create a FlatImage with each FlatChannel
    image_named_tuple = FlatImage(image=[ch_r, ch_g, ch_b])

    post_req = requests.post(
        classify_url,
        image_named_tuple.SerializeToString()
    )

    from visualdocclassification_6_pb2 import ClassifyOut
    of = ClassifyOut()
    of.ParseFromString(post_req.content)

    return of.value



def main():
    from keras.preprocessing.image import load_img, img_to_array
    from keras.applications.resnet50 import preprocess_input
    img = load_img('sample.jpg',
                   target_size=(224, 224))
    # convert the image pixels to a numpy array
    img = img_to_array(img)
    # prepare the image for the VGG model
    img = preprocess_input(img)

    class_pred = classify_doc(img)
    print('Predicted: {}'.format(class_pred))


if __name__ == '__main__':
    main()
